"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMarkdownToVueRenderFn = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const gray_matter_1 = __importDefault(require("gray-matter"));
const lru_cache_1 = __importDefault(require("lru-cache"));
const markdown_1 = require("./markdown/markdown");
const parseHeader_1 = require("./utils/parseHeader");
const slash_1 = require("./utils/slash");
const chalk_1 = __importDefault(require("chalk"));
const debug_1 = __importDefault(require("debug"));
const demo_1 = require("./demo");
const debug = (0, debug_1.default)('vitepress:md');
const cache = new lru_cache_1.default({ max: 1024 });
const includesRE = /<!--\s*@include:\s*(.*?)\s*-->/g;
function createMarkdownToVueRenderFn(srcDir, options = {}, pages, userDefines, isBuild = false) {
    const md = (0, markdown_1.createMarkdownRenderer)(srcDir, options);
    pages = pages.map((p) => (0, slash_1.slash)(p.replace(/\.md$/, '')));
    const userDefineRegex = userDefines
        ? new RegExp(`\\b(${Object.keys(userDefines)
            .map((key) => key.replace(/[-[\]/{}()*+?.\\^$|]/g, '\\$&'))
            .join('|')})`, 'g')
        : null;
    return (src, file, publicDir) => {
        var _a;
        const relativePath = (0, slash_1.slash)(path_1.default.relative(srcDir, file));
        const dir = path_1.default.dirname(file);
        const cached = cache.get(src);
        if (cached) {
            debug(`[cache hit] ${relativePath}`);
            return cached;
        }
        const start = Date.now();
        // resolve includes
        let includes = [];
        src = src.replace(includesRE, (_, m1) => {
            const includePath = path_1.default.join(dir, m1);
            const content = fs_1.default.readFileSync(includePath, 'utf-8');
            includes.push((0, slash_1.slash)(includePath));
            return content;
        });
        const { content, data: frontmatter } = (0, gray_matter_1.default)(src);
        md.realPath = (_a = frontmatter === null || frontmatter === void 0 ? void 0 : frontmatter.map) === null || _a === void 0 ? void 0 : _a.realPath;
        md.urlPath = file;
        let { html, data } = md.render(content);
        const demoData = (0, demo_1.demoPlugin)(html, md);
        html = demoData.html;
        if (isBuild) {
            // avoid env variables being replaced by vite
            html = html
                .replace(/\bimport\.meta/g, 'import.<wbr/>meta')
                .replace(/\bprocess\.env/g, 'process.<wbr/>env');
            // also avoid replacing vite user defines
            if (userDefineRegex) {
                html = html.replace(userDefineRegex, (_) => `${_[0]}<wbr/>${_.slice(1)}`);
            }
        }
        // validate data.links
        const deadLinks = [];
        if (data.links) {
            const dir = path_1.default.dirname(file);
            for (let url of data.links) {
                url = url.replace(/[?#].*$/, '').replace(/\.(html|md)$/, '');
                if (url.endsWith('/'))
                    url += `index`;
                const resolved = (0, slash_1.slash)(url.startsWith('/')
                    ? url.slice(1)
                    : path_1.default.relative(srcDir, path_1.default.resolve(dir, url)));
                if (!pages.includes(resolved) &&
                    !fs_1.default.existsSync(path_1.default.resolve(dir, publicDir, `${resolved}.html`))) {
                    console.warn(chalk_1.default.yellow(`\n(!) Found dead link ${chalk_1.default.cyan(url)} in file ${chalk_1.default.white.dim(file)}`));
                    deadLinks.push(url);
                }
            }
        }
        const pageData = {
            title: inferTitle(frontmatter, content),
            description: inferDescription(frontmatter),
            frontmatter,
            headers: data.headers,
            relativePath,
            // TODO use git timestamp?
            lastUpdated: Math.round(fs_1.default.statSync(file).mtimeMs)
        };
        data.hoistedTags = [demoData.tagsObj];
        const vueSrc = genPageDataCode(data.hoistedTags || [], pageData).join('\n') +
            `\n<template><div>${html}</div></template>`;
        debug(`[render] ${file} in ${Date.now() - start}ms.`);
        const result = {
            vueSrc,
            pageData,
            deadLinks,
            includes
        };
        cache.set(src, result);
        return result;
    };
}
exports.createMarkdownToVueRenderFn = createMarkdownToVueRenderFn;
const scriptRE = /<\/script>/;
const scriptSetupRE = /<\s*script[^>]*\bsetup\b[^>]*/;
const scriptClientRe = /<\s*script[^>]*\bclient\b[^>]*/;
const defaultExportRE = /((?:^|\n|;)\s*)export(\s*)default/;
const namedDefaultExportRE = /((?:^|\n|;)\s*)export(.+)as(\s*)default/;
function genPageDataCode(tags, data) {
    var _a;
    const code = `\nexport const __pageData = ${JSON.stringify(JSON.stringify(data))}`;
    const tagsObj = tags.pop();
    if ((_a = tagsObj.components) === null || _a === void 0 ? void 0 : _a.length) {
        tags.unshift(`<script>${code}\n${(tagsObj.script || []).join(';')}\n
      export default {
      components: {
        ${(tagsObj.components || []).join(', ')}
      }
    }</script>`);
        return tags;
    }
    const existingScriptIndex = tags.findIndex((tag) => {
        return (scriptRE.test(tag) &&
            !scriptSetupRE.test(tag) &&
            !scriptClientRe.test(tag));
    });
    if (existingScriptIndex > -1) {
        const tagSrc = tags[existingScriptIndex];
        // user has <script> tag inside markdown
        // if it doesn't have export default it will error out on build
        const hasDefaultExport = defaultExportRE.test(tagSrc) || namedDefaultExportRE.test(tagSrc);
        tags[existingScriptIndex] = tagSrc.replace(scriptRE, code + (hasDefaultExport ? `` : `\nexport default{}\n`) + `</script>`);
    }
    else {
        tags.unshift(`<script>${code}\nexport default {}</script>`);
    }
    return tags;
}
const inferTitle = (frontmatter, content) => {
    if (frontmatter.home) {
        return 'Home';
    }
    if (frontmatter.title) {
        return (0, parseHeader_1.deeplyParseHeader)(frontmatter.title);
    }
    const match = content.match(/^\s*#+\s+(.*)/m);
    if (match) {
        return (0, parseHeader_1.deeplyParseHeader)(match[1].trim());
    }
    return '';
};
const inferDescription = (frontmatter) => {
    const { description, head } = frontmatter;
    if (description !== undefined) {
        return description;
    }
    return (head && getHeadMetaContent(head, 'description')) || '';
};
const getHeadMetaContent = (head, name) => {
    if (!head || !head.length) {
        return undefined;
    }
    const meta = head.find(([tag, attrs = {}]) => {
        return tag === 'meta' && attrs.name === name && attrs.content;
    });
    return meta && meta[1].content;
};
//# sourceMappingURL=markdownToVue.js.map